
/**
 * Copyright (c) 2003 CA.  All rights reserved.  
 * This software may not be duplicated, disclosed or reproduced in whole or 
 * in part for any purpose except as authorized by the applicable license agreement, 
 * without the express written authorization of CA. All authorized reproductions 
 * must be marked with this language.  
 *
 * TO THE EXTENT PERMITTED BY APPLICABLE LAW, CA PROVIDES THIS 
 * SOFTWARE AS IS WITHOUT WARRANTY OF ANY KIND, INCLUDING 
 * WITHOUT LIMITATION, ANY IMPLIED WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE OR NONINFRINGEMENT.  IN NO EVENT 
 * WILL CA BE LIABLE TO THE END USER OR ANY THIRD PARTY FOR ANY LOSS 
 * OR DAMAGE, DIRECT OR INDIRECT, FROM THE USE OF THIS MATERIAL, 
 * INCLUDING WITHOUT LIMITATION, LOST PROFITS, BUSINESS 
 * INTERRUPTION, GOODWILL, OR LOST DATA, EVEN IF CA IS EXPRESSLY 
 * ADVISED OF SUCH LOSS OR DAMAGE.
 */
/*
*
*              SiteMinder Java DMS API sample program.
*
* Usage: java DmsApiSample [-l]
*
*        -l    Logs the output to a file
*
* (see the classpath options in the supplied
* java-run.bat and java-run.sh scripts)
*/

import java.io.*;
import java.text.*;
import java.util.*;
import java.net.InetAddress;

import com.netegrity.sdk.apiutil.*;
import com.netegrity.sdk.policyapi.*;
import com.netegrity.sdk.dmsapi.*;
import netegrity.siteminder.javaagent.*;

public class DmsApiSample {
    // Logging
    private static PrintWriter logWriter = null;
    private static final String CRLF = System.getProperty("line.separator");
    private static boolean LOGGING_DETAIL = false;
    private static String SECTION_SEP = "";

    // Resource bundle (smjsdksample.properties)
    private static ResourceBundle bundle = null;
    private static final String BUNDLE_NAME = "smjsdksample";

    // Connection to the policy server
    public static SmApiConnection apiConnection = null;

    // Resource and the client IP
    public static String RESOURCE = null;
    public static String CLIENT_IP = null;

    // SiteMinder admin credentials
    public static String ADMIN = null;
    public static String ADMIN_PWD = null;

    // Directory entries that will be created.
    public static String USER_DIR = null;
    public static String LDAP_UID_ATTR = null;
    public static String DIR_ROOT = null;
    public static String ORG_ROOT = null;
    public static String OU_ENG_DN = null;
    public static String OU_DEV_DN = null;
    public static String PEOPLE_ENG_DN = null;
    public static String PEOPLE_DEV_DN = null;
    public static String ORGADMIN_GROUP = null;
    public static String ORGADMIN_GROUP2 = null;
    public static String DEVELOPER = null;
    public static String DEVELOPER_PWD = null;
    public static String USER_DN1 = null;
    public static String USER_DN2 = null;
    public static String USER_DN3 = null;

    static {
	// Initialize resource bundle
	try {
	    bundle = ResourceBundle.getBundle(BUNDLE_NAME);

	    LOGGING_DETAIL = new Boolean(bundle.getString("LOGGING_DETAIL"))
		    .booleanValue();
	    SECTION_SEP = bundle.getString("SECTION_SEPARATOR");
	    RESOURCE = bundle.getString("RESOURCE_NAME");
	    CLIENT_IP = bundle.getString("AGENT_IP");
	    ADMIN = bundle.getString("ADMIN_NAME");
	    ADMIN_PWD = bundle.getString("ADMIN_PWD");
	    USER_DIR = bundle.getString("USER_DIR_NAME");
	    LDAP_UID_ATTR = bundle.getString("LDAP_UID_ATTR");


	    DIR_ROOT = bundle.getString("LDAP_USERDIR_SRCH_ROOT");
	    ORG_ROOT = bundle.getString("DMSAPI_ORG_ROOT") + "," + DIR_ROOT;
	    OU_ENG_DN = bundle.getString("DMSAPI_OU_ENG_DN") + "," + ORG_ROOT;
	    OU_DEV_DN = bundle.getString("DMSAPI_OU_DEV_DN") + "," + OU_ENG_DN;
	    PEOPLE_ENG_DN = bundle.getString("DMSAPI_PEOPLE_DN") + ","
		    + OU_ENG_DN;
	    PEOPLE_DEV_DN = bundle.getString("DMSAPI_PEOPLE_DN") + ","
		    + OU_DEV_DN;
	    ORGADMIN_GROUP = bundle.getString("DMSAPI_ORGADMIN_GROUP") + ","
		    + OU_ENG_DN;
	    ORGADMIN_GROUP2 = bundle.getString("DMSAPI_ORGADMIN_GROUP") + ","
		    + OU_DEV_DN;
	    DEVELOPER = bundle.getString("DMSAPI_DEVELOPER");
	    DEVELOPER_PWD = bundle.getString("DMSAPI_DEVELOPER_PWD");
	    USER_DN1 = LDAP_UID_ATTR + "=" + bundle.getString("DMSAPI_USER1")
		    + "," + PEOPLE_ENG_DN;
	    USER_DN2 = LDAP_UID_ATTR + "=" + bundle.getString("DMSAPI_USER2")
		    + "," + PEOPLE_ENG_DN;
	    USER_DN3 = LDAP_UID_ATTR + "=" + bundle.getString("DMSAPI_USER3")
		    + "," + PEOPLE_ENG_DN;
	} catch (MissingResourceException mre) {
	    LogFatal("Failed to load the resource bundle " + BUNDLE_NAME);
	} catch (ClassCastException cce) {
	    LogFatal("Failed to load the resource bundle " + BUNDLE_NAME);
	} catch (NullPointerException npe) {
	    LogFatal("Failed to load the resource bundle.");
	}
    }

    public static void main(String[] args) {
	// Parse command line arguments
	for (int i = 0; i < args.length; ++i) {
	    // '-l' enables logging to a file
	    if ((args[i].trim()).equalsIgnoreCase("-l")) {
		try {
		    String logFile = bundle.getString("LOGFILE_NAME");
		    logWriter = new PrintWriter(
			    new BufferedWriter(new FileWriter(logFile)));
		} catch (IOException ioe) {
		    LogException(ioe);
		    logWriter = null; // no logging
		}

		break;
	    }
	}

	Log(CRLF + CRLF + bundle.getString("DMSAPI_HEADER") + CRLF);
	DmsApiSample dmsApiSample = new DmsApiSample();

	// Connect to the policy server
	String agentIP = bundle.getString("AGENT_IP");
	String agentName = bundle.getString("AGENT_NAME");
	String agentSecret = bundle.getString("AGENT_SECRET");

	AgentAPI agentapi = new AgentAPI();
	ServerDef serverdef = new ServerDef();
	serverdef.serverIpAddress = bundle.getString("PS_IP");
	serverdef.connectionMin = new Integer(bundle.getString("PS_CONMIN"))
		.intValue();
	serverdef.connectionMax = new Integer(bundle.getString("PS_CONMAX"))
		.intValue();
	serverdef.connectionStep = new Integer(bundle.getString("PS_CONSTEP"))
		.intValue();
	serverdef.timeout = new Integer(bundle.getString("PS_TIMEOUT"))
		.intValue();
	serverdef.authenticationPort = new Integer(
		bundle.getString("PS_AUPORT")).intValue();
	serverdef.authorizationPort = new Integer(bundle.getString("PS_AZPORT"))
		.intValue();
	serverdef.accountingPort = new Integer(bundle.getString("PS_ACPORT"))
		.intValue();

	InitDef initdef = new InitDef(agentName, agentSecret, false, serverdef);

	Log(CRLF + bundle.getString("AGENTAPI_PS_IP")
		+ serverdef.serverIpAddress);
	Log(CRLF + bundle.getString("AGENTAPI_AGENT_IP") + agentIP);
	Log(CRLF + bundle.getString("AGENTAPI_AGENT_NAME") + agentName);
	Log(CRLF + bundle.getString("AGENTAPI_AGENT_SECRET") + agentSecret);

	int retcode = agentapi.init(initdef);

	if (retcode != AgentAPI.SUCCESS) {
	    LogFatal(bundle.getString("FAILED_CONNECT"));
	}

	apiConnection = new SmApiConnection(agentapi);

	// Run the main tests
	dmsApiSample.useDirectoryContext(false);

	Log(CRLF + CRLF + bundle.getString("SAMPLE_FOOTER") + CRLF);

	if (logWriter != null) {
	    logWriter.close();
	}
    }

    // This function call uses the directory context
    public void useDirectoryContext(boolean bDelete) {
	Log(CRLF + CRLF + bundle.getString("DIRCONTEXT_HEADER") + CRLF);

	// First get the directory context
	SmDmsDirectoryContext directoryContext = getDirectoryContext();

	if (directoryContext == null) {
	    return;
	}

	// directoryContext contains a valid agent API connection
	// and administrator session spec.
	SmDmsDirectory dmsDirectory = directoryContext.getDmsDirectory();

	if (dmsDirectory == null) {
	    return;
	}

	if (bDelete == false) {
	    searchCallsODBC(dmsDirectory);
	} 
    }

    public SmDmsDirectoryContext getDirectoryContext() {
	try {
	    SmApiResult result = new SmApiResult();

	    // Session object using default static Agent API handle.
	    SmApiSession apiSession = new SmApiSession(apiConnection);
	    boolean loginResult = adminLogin(apiSession);

	    if (!loginResult) {
		return null;
	    }

	    // Get a list of user directories the admin can manage.
	    SmPolicyApi policyApi = new SmPolicyApiImpl(apiSession);
	    Log(CRLF + bundle.getString("DMSAPI_GET_ADMINDIRS"));
	    Vector userDirs = new Vector();

	    // Returns a list of directory names.
	    result = policyApi.getAdminUserDirs(ADMIN, userDirs);
	    printObject(userDirs, result, false);

	    // Check if the USER_DIR can be found in the list.
	    SmUserDirectory userDir = null;

	    for (int i = 0; i < userDirs.size(); ++i) {
		String dir = (String) userDirs.get(i);

		if (dir.equals(USER_DIR)) {
		    Log(CRLF + bundle.getString("DMSAPI_GET_USERDIR"));
		    userDir = new SmUserDirectory(USER_DIR);
		    result = policyApi.getUserDirectory(USER_DIR, userDir);
		    printObject(userDir, result, false);
		}
	    }

	    if (userDir == null) {
		LogError(bundle.getString("DMSAPI_FAILED_USERDIR"));
		return null;
	    }

	    SmDmsApi dmsApi = new SmDmsApiImpl(apiSession);
	    SmDmsDirectoryContext dirContext = new SmDmsDirectoryContext();
	    result = dmsApi.getDirectoryContext(userDir, new SmDmsConfig(),
		    dirContext);

	    // The dmsDir contains both session and agent connection.
	    // At this time it contains SiteMinder administrator session.
	    // From here if you continue using the same SiteMinder
	    // administrator.
	    return dirContext;
	} catch (SmApiException apiException) {
	    LogException(apiException);
	}

	return null;
    }


    public void searchCallsODBC(SmDmsDirectory dmsDir) {
	SmApiResult result = new SmApiResult();
	String SRCH_FILTER = "from SmGroup";
	String[] prop_new_ODBC = { "Name", "'Group' as Class" };
	String DIR_ROOT = "DUMMY_DIR_ROOT";// NOT applicable for ODBC

	try {

	    String directoryType = dmsDir.getDirType();
	    Log(CRLF + "Directory Type is:" + directoryType + CRLF);

	    SmDmsOrganization org = dmsDir.newOrganization(DIR_ROOT);
	    SmDmsSearch search = new SmDmsSearch(SRCH_FILTER, DIR_ROOT);
	    search.setScope(1);
	    search.setNextItem(0);
	    search.setMaxItems(5000);
	    search.setPreviousItem(0);
	    search.setPropertyNames(prop_new_ODBC);
	    search.setMaxResults(5000);

	    result = org.search(search, 1); // search forward
	    Vector vsearch = search.getResults();

	    SmDmsSearchResultParams searchParams = (SmDmsSearchResultParams) vsearch
		    .firstElement();
	    vsearch.removeElementAt(0);
	    Log(CRLF + "RESULT:");
	    for (int i = 0; i < vsearch.size(); ++i) {
		SmDmsObject dmsObj = (SmDmsObject) vsearch.elementAt(i);
		Log(CRLF + dmsObj );
	    }

	    vsearch.removeAllElements();
	  
	} catch (SmApiException apiException) {
	    System.out.println(apiException.getMessage());
	}
    }

   
    public RealmDef agentApiIsProtected(AgentAPI agentApi) {
	// Agent API Login call
	ResourceContextDef resourceContext = new ResourceContextDef("", "",
		RESOURCE, "GET");
	RealmDef realmContext = new RealmDef();

	Log(CRLF + bundle.getString("DMSAPI_ISPROTECTED"));
	int agentResult = agentApi.isProtected(CLIENT_IP, resourceContext,
		realmContext);

	if (agentResult != AgentAPI.YES) {
	    Log(bundle.getString("STATUS_NOK"));
	    return null;
	}

	Log(bundle.getString("STATUS_OK"));

	if (LOGGING_DETAIL == true) {
	    Log(CRLF + bundle.getString("DMSAPI_REALM_NAME")
		    + realmContext.name);
	    Log(CRLF + bundle.getString("DMSAPI_REALM_OID") + realmContext.oid);
	}

	return realmContext;
    }

    public String agentApiLogin(AgentAPI agentApi, String user,
	    String password) {
	// Agent API Login call
	ResourceContextDef resourceContext = new ResourceContextDef("", "",
		RESOURCE, "GET");
	RealmDef realmContext = agentApiIsProtected(agentApi);

	if (realmContext == null) {
	    return null;
	}

	// Now let's do the authentication
	UserCredentials userCreds = new UserCredentials(user, password);
	SessionDef sessionContext = new SessionDef();
	AttributeList al = new AttributeList();
	Log(CRLF + bundle.getString("DMSAPI_LOGIN"));

	int agentResult = agentApi.login(CLIENT_IP, resourceContext,
		realmContext, userCreds, sessionContext, al);

	if (agentResult != AgentAPI.YES) {
	    Log(bundle.getString("STATUS_NOK"));
	    Log(CRLF + pad("Result Code") + agentResult);
	    Log(CRLF + pad("Reason Code") + sessionContext.reason);
	    return null;
	}

	Log(bundle.getString("STATUS_OK"));
	return sessionContext.spec;
    }

    public boolean adminLogin(SmApiSession apiSession) {
	// SiteMinder admin login.
	try {
	    InetAddress address = InetAddress.getLocalHost();

	    SmApiResult result = apiSession.login(ADMIN, ADMIN_PWD, address, 0);

	    if (!result.isSuccess()) {
		printObject(null, result, false);
		return false;
	    }
	} catch (java.net.UnknownHostException uhe) {
	    LogException(uhe);
	    return false;
	} catch (SmApiException apiException) {
	    LogException(apiException);
	    return false;
	}

	return true;
    }

    public static void printDmsContext(SmDmsContext dmsContext,
	    SmApiResult result) {
	if (!result.isSuccess()) {
	    Log(CRLF + bundle.getString("DMSAPI_FAILED_DMS"));
	    Log(result.toString());
	    return;
	}

	SmRealm dmsRealm = dmsContext.getRealm();
	Log(CRLF + bundle.getString("DMSAPI_DMS_REALM"));
	printObject(dmsRealm, result, false);

	SmSelfReg selfReg = dmsContext.getSelfReg();
	Log(CRLF + bundle.getString("DMSAPI_SELFREG"));
	printObject(selfReg, result, false);

	SmUserDirectory dir = dmsContext.getDmsDirectory().getUserDirectory();
	Log(CRLF + bundle.getString("DMSAPI_USERDIR"));
	printObject(dir, result, false);
    }

    private static void printObject(Object obj, final SmApiResult result,
	    boolean bMustFail) {
	if (bMustFail == false) {
	    if (!result.isSuccess()) {
		Log(bundle.getString("STATUS_NOK"));
	    } else {
		Log(bundle.getString("STATUS_OK"));
	    }
	} else {
	    if (!result.isSuccess()) {
		Log(bundle.getString("STATUS_OK"));
	    } else {
		Log(bundle.getString("STATUS_NOK"));
	    }
	}

	if (LOGGING_DETAIL == true) {
	    Log(CRLF + pad("Result Code") + result.toString());

	    if (obj != null) {
		if (obj instanceof com.netegrity.sdk.policyapi.SmObject) {
		    SmObject SmObj = (SmObject) obj;
		    Hashtable properties = new Hashtable(25);
		    SmObj.writeProperties(properties);
		    obj = properties;
		} else if (obj instanceof com.netegrity.sdk.dmsapi.SmDmsObject) {
		    SmDmsObject dmsObj = (SmDmsObject) obj;
		    obj = dmsObj.getAttributes();
		}

		if (obj instanceof java.util.Hashtable) {
		    Enumeration ekeys = ((Hashtable) obj).keys();
		    Enumeration evalues = ((Hashtable) obj).elements();
		    boolean bFirstElem = true;

		    while (evalues.hasMoreElements()) {
			if (!bFirstElem) {
			    Log(CRLF + "\t\t\t\t\t");
			}

			Log(ekeys.nextElement() + "=" + evalues.nextElement());
			bFirstElem = false;
		    }
		} else if (obj instanceof java.util.Vector) {
		    Enumeration evalues = ((Vector) obj).elements();
		    boolean bFirstElem = true;

		    while (evalues.hasMoreElements()) {
			if (!bFirstElem) {
			    Log(CRLF + "\t\t\t\t\t");
			}

			Log(evalues.nextElement().toString());
			bFirstElem = false;
		    }
		}
	    }
	}
    }

    private static String pad(Object object) {
	String message = object.toString();
	String padding = bundle.getString("DMSAPI_PADDING");
	String ret = padding.substring(0, 1) + message;

	if (message.length() < padding.length() - 3) {
	    ret += padding.substring(message.length() + 1);
	} else {
	    ret += padding.substring(padding.length() - 2);
	}

	return ret;
    }

    private static void Log(String log) {
	System.out.print(log);

	if (logWriter != null) {
	    logWriter.print(log);
	}
    }

    private static void LogError(String errLog) {
	Log(CRLF + "ERROR: " + errLog);
    }

    private static void LogException(Exception e) {
	Log(CRLF + "EXCEPTION: " + e.getMessage());
    }

    private static void LogFatal(String fatalLog) {
	Log(CRLF + "FATAL: " + fatalLog);

	if (logWriter != null) {
	    logWriter.close();
	}

	System.exit(-1);
    }
}

// EOF DmsApiSample.java
